#include "contiki.h"
#include "lib/random.h"
#include "net/rime/rime.h"
#include "net/rime/collect.h"
#include "dev/leds.h"
#include "dev/button-sensor.h"
#include "dev/serial-line.h"
#include "net/netstack.h"
#include <stdio.h>
#include <stdlib.h>

#define add_bytes 1
// one additional byte to allow coding sequence numbers beyond 255

static struct collect_conn tc;

/*---------------------------------------------------------------------------*/
PROCESS(sensevid_collect_process, "SenseVid test collect process");
AUTOSTART_PROCESSES(&sensevid_collect_process);
/*---------------------------------------------------------------------------*/
static void recv(const linkaddr_t *originator, uint8_t seqno, uint8_t hops){
  char * seqptr = packetbuf_dataptr();
  uint16_t seq = seqptr[0];
  printf("%d payload %d\n" , seqno+1+(seq<<8), packetbuf_datalen()-add_bytes);
  
}
/*---------------------------------------------------------------------------*/
static const struct collect_callbacks callbacks = { recv };
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
PROCESS_THREAD(sensevid_collect_process, ev, data){
  static struct etimer et;
  
  PROCESS_BEGIN();

  collect_open(&tc, 130, COLLECT_ROUTER, &callbacks);

 
  if(linkaddr_node_addr.u8[0] == 1 && linkaddr_node_addr.u8[1] == 0) {
    collect_set_sink(&tc, 1);
  }

  /* Allow some time for the network to settle. */
  etimer_set(&et, 60 * CLOCK_SECOND);
  PROCESS_WAIT_UNTIL(etimer_expired(&et));

  if(linkaddr_node_addr.u8[0] == 2 && linkaddr_node_addr.u8[1] == 0) {
    printf("Source : READY TO SEND ...\n");
    static uint16_t seq=0;
    
    while(1) {
      
      static linkaddr_t oldparent;
      const linkaddr_t *parent;
      uint8_t size;
      
      PROCESS_YIELD();
      if(ev == serial_line_event_message) {

	size = atoi(strtok(data, " "));
        data = strtok(NULL, " ");
        seq = atoi(strtok(data, " "));
        printf("received line: %d seqno %d i.e. %d\n",size, seq, seq >> 8);
	 
	packetbuf_clear(); 
	char * msg = packetbuf_dataptr();
	msg[0] = seq >> 8;
	packetbuf_set_datalen(size+add_bytes);

	collect_send(&tc, 15); 
	
      }
      
     
      parent = collect_parent(&tc);
      if(!linkaddr_cmp(parent, &oldparent))
    	linkaddr_copy(&oldparent, parent);
      
    }
  }
  PROCESS_END();
}
